# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import gocept.directory.interfaces
import grok
import os.path
import yaml


grok.context(gocept.directory.interfaces.IDirectory)


class AccessRing0(grok.Permission):
    grok.name('gocept.directory.AccessRing0')


class Ring0(grok.XMLRPC):

    # XXX Requires changes to our clients.
    # grok.require(AccessRing0)

    def __init__(self, context, request):
        self.dbdir = context.paltry
        self.node_db = os.path.join(self.dbdir, 'nodes')
        self.passwd_db = os.path.join(self.dbdir, 'passwords')
        self.people_db = os.path.join(self.dbdir, 'people', 'people.yml')
        self.groups_db = os.path.join(self.dbdir, 'people', 'groups.yml')
        self.location_db = os.path.join(self.dbdir, 'locations')

    @property
    def __nodes(self):
        for name in os.listdir(self.node_db):
            if name.startswith('.'):
                continue
            node = yaml.load(open(os.path.join(self.node_db, name)))
            node['name'] = name
            # DEPRECATED: provide node['fqdn'] for backward compatibility
            node['fqdn'] = name
            yield node

    def list_virtual_machines(self):
        result = []
        for node in self.__nodes:
            if node['parameters']['machine'] != 'virtual':
                continue
            result.append(node)
        return result

    def list_nodes(self):
        result = []
        for node in self.__nodes:
            result.append(node)
        return result

    def lookup_node(self, lookup_name):
        for node in self.__nodes:
            if node['name'] != lookup_name:
                continue
            return node
        raise KeyError(lookup_name)

    def get_password(self, service):
        if not service in os.listdir(self.passwd_db):
            raise KeyError(service)
        return open(os.path.join(self.passwd_db, service)).read().strip()

    def list_people(self):
        return yaml.load(open(self.people_db))

    def list_groups(self):
        return yaml.load(open(self.groups_db))

    def lookup_location(self, name):
        return yaml.load(open(os.path.join(self.location_db, name+'.yaml')))

    def list_networks(self):
        networks = []
        for location in os.listdir(self.location_db):
            if location.startswith('.'):
                continue
            location = location.replace('.yaml', '')
            for network in self.lookup_location(location)['networks'].values():
                networks.extend(network.values())
        return networks
