# vim:fileencoding=utf-8
# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import gocept.directory.testing
import os.path
import unittest
import xmlrpclib
import zope.app.testing.functional
import zope.app.testing.xmlrpc


class TestRing0(zope.app.testing.functional.FunctionalTestCase):

    layer = gocept.directory.testing.DataLayer

    def setUp(self):
        self.directory = self.getRootFolder()['directory']
        self.directory.paltry = os.path.join(
            os.path.dirname(gocept.directory.__file__), 'testdb')
        self.rpc = zope.app.testing.xmlrpc.ServerProxy('http://localhost/directory/')

    def test_password(self):
        self.assertEquals('nenpotgaf', self.rpc.get_password('ldap'))
        self.assertRaises(xmlrpclib.Fault, self.rpc.get_password, 'foobar')

    def test_people(self):
        people = self.rpc.list_people()
        self.assertEquals(40, len(people))
        self.assertEquals({
            'mail': 'ct@gocept.com',
            'pager': '+49 179 7808366',
            'uid': 'ctheune',
            'cn': 'Christian Theune'}, people['ctheune'])

    def test_groups(self):
        groups = self.rpc.list_groups()
        self.assertEquals(17, len(groups))
        self.assertEquals(['zagy', 'ctheune', 'nilo', 'ckauhaus'],
                          groups['admins'])

    def test_lookup_node(self):
        self.assertEquals({
            'classes': ['role::puppetmaster', 'role::nagios',
                        'role::portage_binhost'],
            'name': 'stampy',
            'fqdn': 'stampy',
            'parameters': {
                'environment': 'testing',
                'mac_fe': '00:15:17:91:d2:f0',
                'contacts': 'ckauhaus, ctheune',
                'ip4_sto': '10',
                'machine': 'physical',
                'ip4_fe': '3',
                'mac_srv': '00:1e:c9:ad:4a:a0',
                'location': 'whq',
                'ip4_srv': '10',
                'mac_ipmi': '00:1e:c9:ad:4a:a4',
                'mac_sto': '00:1e:c9:ad:4a:a2',
                'fqdn_fe': 'monitor.whq.gocept.com',
                'ip4_ipmi': '9'}},
            self.rpc.lookup_node('stampy'))
        self.assertRaises(xmlrpclib.Fault, self.rpc.lookup_node, 'foobar')

    def test_list_virtual_machines(self):
        vms = self.rpc.list_virtual_machines()
        self.assertEquals(1, len(vms))
        self.assertEquals('test00.gocept.net', vms[0]['name'])

    def test_lookup_location(self):
        self.assertEquals(
            {'networks': {'sto': {'ipv4': '172.20.4.0/24',
                                  'ipv6': '2001:470:9aaf:44::/64'},
                          'srv': {'ipv4': '172.20.3.0/24',
                                  'ipv6': '2001:470:9aaf:43::/64'},
                          'fe': {'ipv4': '172.20.2.0/24', 'ipv6':
                                 '2001:470:9aaf:42::/64'},
                          'mgm': {'ipv4': '172.20.1.0/24',
                                  'ipv6':
                                  '2001:470:9aaf:41::/64'}}},
            self.rpc.lookup_location('jamaica'))

    def test_list_networks(self):
        self.assertEquals(
            ['231.20.4.0/24', '2002:470:9aaf:44::/64', '231.20.3.0/24',
             '2002:470:9aaf:43::/64', '231.20.2.0/24', '2002:470:9aaf:42::/64',
             '231.20.1.0/24', '2002:470:9aaf:41::/64', '172.20.4.0/24',
             '2001:470:9aaf:44::/64', '172.20.3.0/24', '2001:470:9aaf:43::/64',
             '172.20.2.0/24', '2001:470:9aaf:42::/64', '172.20.1.0/24',
             '2001:470:9aaf:41::/64'],
            self.rpc.list_networks())

    def test_list_nodes(self):
        self.assertEquals(
            [{'classes': ['role::puppetmaster', 'role::nagios',
                          'role::portage_binhost'],
              'name': 'stampy',
              'parameters': {'environment': 'testing',
                             'mac_fe': '00:15:17:91:d2:f0',
                             'contacts': 'ckauhaus, ctheune',
                             'ip4_sto': '10',
                             'machine': 'physical',
                             'ip4_fe': '3',
                             'mac_srv': '00:1e:c9:ad:4a:a0',
                             'location': 'whq',
                             'ip4_srv': '10',
                             'mac_ipmi': '00:1e:c9:ad:4a:a4',
                             'mac_sto': '00:1e:c9:ad:4a:a2',
                             'fqdn_fe': 'monitor.whq.gocept.com',
                             'ip4_ipmi': '9'},
              'fqdn': 'stampy'},
             {'classes': ['role::kvm_host'],
              'name': 'otto',
              'parameters': {'environment': 'feature_vm_starting',
                             'mac_fe': '00:15:17:91:cd:0c',
                             'contacts': 'ckauhaus, ctheune',
                             'ip4_sto': '11',
                             'machine': 'physical',
                             'mac_srv': '00:1e:c9:b7:7b:64',
                             'location': 'whq',
                             'ip4_srv': '12',
                             'mac_ipmi': '00:1e:c9:b7:7b:68',
                             'mac_sto': '00:1e:c9:b7:7b:66',
                             'ip4_ipmi': '11'},
              'fqdn': 'otto'},
             {'classes': ['role::customerproject'],
              'name': 'test00.gocept.net',
              'parameters': {'environment': 'testing',
                             'mac_fe': '02:00:00:00:02:00',
                             'contacts': 'ckauhaus, ctheune',
                             'machine': 'virtual',
                             'ip4_fe': '195.62.106.16/26',
                             'rootdev': '/dev/etherd/1.0',
                             'mac_srv': '02:00:00:00:01:00',
                             'location': 'rzl',
                             'memory': 1024,
                             'ip4_srv': '195.62.106.15/26',
                             'id': 1},
              'fqdn': 'test00.gocept.net'}],
            self.rpc.list_nodes())


def test_suite():
    return unittest.makeSuite(TestRing0)
