# Copyright (c) 2008-2009 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id: testing.py 16250 2009-06-17 13:29:20Z ctheune $

import os
import zope.app.container.interfaces
import zope.app.testing.functional
import zope.component
import zope.event
import zope.lifecycleevent
import zope.testing
from zope.testing import doctest

import gocept.directory.interfaces
import gocept.directory.customer
import gocept.directory.person
import gocept.directory.resourcegroup
import gocept.directory.useraccount


DataLayer = zope.app.testing.functional.ZCMLLayer(
    os.path.join(os.path.dirname(__file__), 'ftesting.zcml'),
    __name__, 'DataLayer', allow_teardown=True)


def add_ftest(suite, filename):
    add_test(suite, filename,
             zope.app.testing.functional.FunctionalDocFileSuite,
             DataLayer)


def add_utest(suite, filename):
    add_test(suite, filename, zope.testing.doctest.DocFileSuite, None)


def add_test(suite, filename, suite_class, layer):
    """Add tests found in filename to test suite."""
    test = suite_class(
        filename,
        optionflags=(doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE |
                     doctest.INTERPRET_FOOTNOTES))
    if layer:
        test.layer = layer
    suite.addTest(test)


def set_site(func):
    "Decorator which does the set-site-dance."
    def decorated(*args, **kw):
        old_site = zope.app.component.hooks.getSite()
        site = zope.app.testing.functional.getRootFolder()['directory']
        try:
            zope.app.component.hooks.setSite(site)
            return func(*args, **kw)
        finally:
            zope.app.component.hooks.setSite(old_site)
    return decorated


def create_helper(container, obj):
    zope.event.notify(zope.lifecycleevent.ObjectCreatedEvent(obj))
    chooser = zope.app.container.interfaces.INameChooser(container)
    name = chooser.chooseName('', obj)
    container[name] = obj
    return container[name]


@set_site
def create_person(first_name, last_name):
    container = zope.component.getUtility(
        gocept.directory.interfaces.IPersons)
    person = gocept.directory.person.Person(first_name, last_name)
    return create_helper(container, person)


@set_site
def create_customer(title):
    container = zope.component.getUtility(
        gocept.directory.interfaces.ICustomers)
    customer = gocept.directory.customer.Customer(title=title)
    return create_helper(container, customer)


@set_site
def create_resource_group(customer, title, group, gid):
    container = zope.component.getUtility(
        gocept.directory.interfaces.IResourceGroups, context=customer)
    group = gocept.directory.resourcegroup.ResourceGroup(group, title)
    group.gid=gid
    return create_helper(container, group)


@set_site
def create_user_account(username, owner, password=u''):
    container = zope.component.getUtility(
        gocept.directory.interfaces.IUnixAccounts)
    account = gocept.directory.useraccount.UserAccount(username, password)
    account.owner = owner
    return create_helper(container, account)
